from django.db import models
from django.utils.translation import gettext as _
from start.models import Searchable

import logging
logger = logging.getLogger(__name__)

import markdown
ARTICLE_DIR = "/app/blog/data/articles"
EXTENSIONS = [
    "extra",
    "admonition",
    "codehilite",
    "meta",
    "toc"
]
MD = markdown.Markdown(extensions=EXTENSIONS)

class Category(models.Model):
    """
    A category of blog posts

    Name not translated because it would make i18n in urls and Searchables specifically a pain.
    Maybe some day it would be cool if these were Searchable
    """
    name= models.CharField(max_length=50)
    slug = models.SlugField()

    class Meta:
        verbose_name = _("Category")
        verbose_name_plural = _("Categories")

    def __str__(self):
        return f"{{<{self.__class__.__name__}>\"{self.name}\"}}"

class BlogPost(Searchable):
    """
    Should contain a blogpost
    """
    body_en = models.TextField(default="No english translation yet.")
    body_de = models.TextField(default="Bis jetzt keine deutsche Übersetzung.")
    category = models.ForeignKey(Category, on_delete=models.SET_NULL, null=True)
    thumbnail = models.ImageField(blank=True, upload_to="img/thumbnails")
    featured = models.BooleanField(default=False)
    slug = models.SlugField()


    def regenerate(self):
        """
        regenerate a object

        Implements the abstract method of Searchable
        """
        logger.info(f"regenerating {self.__class__.__name__} object: {self}")
        # url stuff
        self.suburl = f"/blog/{self.category.name}/{self.slug}"

        # load from markdown
        self.regenerate_markdown()

        self.save()

    def regenerate_markdown(self):
        """
        generate an article fromm it's original markdown file
        """
        logger.info(f"regenerating article from markdown for: {self}")
        try:
            f_en = open(f"{ARTICLE_DIR}/en-{self.slug}.md")

            body_en: str = f_en.read()

            html_en: str = MD.convert(body_en)
            try:
                meta_en = MD.Meta
                # FIXME: some fields take the brackets and quotes with them
                self.title_en = meta_en["title"]
                self.subtitle_en = meta_en["subtitle"]
                self.desc_en = meta_en["desc"]
                # TODO: parse date from markdown
                self.featured = meta_en["featured"] == "True"
                self.public = meta_en["public"] == "True"
                # TODO: parse keywords from markdown
                # TODO: parse category from markdown

                # if keyword or category do not exist, create them
                # I suppose
            except Exception as e:
                logger.warning(f"could not generate metadata {self.slug} from markdown: {e}")

            self.body_en = html_en
        except FileNotFoundError as e:
            # TODO: mark as untranslated
            pass
        except Exception as e:
            logger.warning(f"could not generate article {self.slug} from markdown: {e}")
            print(e)
        try:
            f_de = open(f"{ARTICLE_DIR}/de-{self.slug}.md")

            body_de: str = f_de.read()

            html_de: str = MD.convert(body_de)
            try:
                meta_de = MD.Meta
                # FIXME: some fields take the brackets and quotes with them
                self.title_de = meta_de["title"]
                self.subtitle_de = meta_de["subtitle"]
                self.desc_de = meta_de["desc"]
                # TODO: parse date from markdown
                self.featured = meta_de["featured"] == "True"
                self.public = meta_de["public"] == "True"
                # TODO: parse keywords from markdown
                # TODO: parse category from markdown

                # if keyword or category do not exist, create them
                # I suppose
            except Exception as e:
                logger.warning(f"could not generate metadata {self.slug} from markdown: {e}")

            self.body_de = html_de
        except FileNotFoundError as e:
            # TODO: mark as untranslated
            pass
        except Exception as e:
            logger.warning(f"could not generate article {self.slug} from markdown: {e}")
            print(e.__class__)
            print(e)

    class Meta:
        verbose_name = _("blog post")
        verbose_name_plural = _("blog posts")
